/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008-2009  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.Collection;
import java.util.List;

/** Annotation for DAO finder methods. */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface Finder {
  /**
   * Use this parameter to specify named query name.
   * @return named query name.
   */
  String queryName() default "";

  /**
   * Use this parameter to specify HQL query.
   * @return HQL query.
   */
  String query() default "";

  /**
   * Use this parameter to specify SQL native query name.
   * @return SQL query name.
   */
  String sqlQueryName() default "";

  /**
   * Use this parameter to specify SQL native query.
   * @return SQL query.
   */
  String sqlQuery() default "";

  /**
   * Use this parameter if finder method wants to return custom collection type.
   * <p/>
   * The only requirements for specified type are: no-arg constructor and implemented {@link
   * Collection#addAll(java.util.Collection)} method.
   * <p/>
   * Example:
   * <pre>
   * &#64;Finder(query = "from Book where id in (:ids)", returnAs = HashSet.class)
   * Set&lt;Book&gt; findByIds(@Named("ids") Set&lt;Long&gt; ids);
   * </pre>
   * @return result collection type.
   */
  Class<? extends Collection> returnAs() default List.class;
}
