/**
 * AutoDAO - Generic DAO on steroids implementation for Java.
 * Copyright (C) 2008-2009  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.autodao;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Implemented by AutoDAO implementations.
 * <p/>
 * NOT FOR PUBLIC USE.
 */
public interface FinderExecutor {

  @NotNull
  <T> List<T> executeListFinder(@NotNull final Method method,
                                @NotNull final Map<String, ?> namedArgs,
                                @Nullable final Integer limit,
                                @Nullable final Integer offset);

  /**
   * Execute a finder method with the appropriate arguments.
   * @param method      query method.
   * @param indexedArgs arguments for query method
   * @param limit       query limit. Can be <code>null</code>. See {@link Limit}.
   * @param offset      query offset. Can be <code>null</code>. See {@link Offset}.
   * @return list of found objects or empty list.
   */
  @NotNull
  <T> List<T> executeListFinder(@NotNull final Method method,
                                @NotNull final Object[] indexedArgs,
                                @Nullable final Integer limit,
                                @Nullable final Integer offset);

  @Nullable
  <T> T executeSingleFinder(@NotNull final Method method, @NotNull final Map<String, ?> namedArgs);

  /**
   * Same as {@link #executeListFinder(Method, Object[], Integer, Integer)} but for single result.
   * @param method      query method.
   * @param indexedArgs arguments for query method.
   * @return found object or <code>null</code>
   */
  @Nullable
  <T> T executeSingleFinder(@NotNull final Method method, @NotNull final Object[] indexedArgs);
}
